import java.util.*;
import java.awt.Color;

public enum Planet
    implements Runnable, Point2D
{
  MERCURY("Mercury", 3.303e+23, 2.4397e6, 5.8e+10, 48270, Color.RED),
      VENUS("Venus", 4.869e+24, 6.0518e6, 1.09e+11, 33789, Color.LIGHT_GRAY),
      EARTH("Earth", 5.976e+24, 6.37814e6, 1.5e11, 28962, Color.BLUE),
      MARS("Mars", 6.421e+23, 3.3972e6, 2.28e11, 24135, Color.RED),
      JUPITER("Jupiter", 1.9e+27, 7.1492e7, 7.78e11, 12872, Color.ORANGE),
      SATURN("Saturn", 5.688e+26, 6.0268e7, 1.4e12, 9654, Color.YELLOW),
      URANUS("Uranus", 8.686e+25, 2.5559e7, 2.9e12, 6436, Color.PINK),
      NEPTUNE("Neptune", 1.024e+26, 2.4746e7, 4.5e12, 4827, Color.GREEN),
      PLUTO("Pluto", 1.27e+22, 1.137e6, 5.9e12, 3218, Color.GRAY);

  public static final double G = 6.67300E-11;
  public static final long SCALE = 525600;
  
  private final double m_mass; // in kilograms
  private final double m_radius; // in meters
  private double m_distance;
  private double m_speed;
  private double m_angle;
  private String m_name;
  private Color m_color;

  private Thread m_thread;
  
  public Color getColor()
  {
	return m_color;  
  }
  
  private static Boolean m_isRunning = false;

  public List<PlanetListener> m_listeners = new ArrayList<PlanetListener>();

  public double getDistance()
  {
	  return m_distance;
  }
  
  public void addListener(PlanetListener l)
  {
    m_listeners.add(l);
  }

  public void removeListener(PlanetListener l)
  {
    m_listeners.remove(l);
  }

  private void dispatchMove()
  {
    for (PlanetListener l : m_listeners)
      l.planetMoved(this);
  }

  Planet(String name, double mass, double radius, double distance, double speed, Color c)
  {
	m_thread = new Thread(this);
    m_name = name;
    m_mass = mass;
    m_radius = radius;
    m_angle = 0;
    m_speed = speed;
    m_distance = distance;
    m_color = c;
  }

  public String getName()
  {
    return m_name;
  }

  public String toString()
  {
    return m_name + " (r:" + (long)m_radius + ", d:" + (long)m_distance + ")  " +
        Math.round(m_angle);
  }

  public double getMass()
  {
    return m_mass;
  }

  public double getRadius()
  {
    return m_radius;
  }

  public double getSurfaceGravity()
  {
    return G * m_mass / (m_radius * m_radius);
  }

  public double getSurfaceWeight(double otherMass)
  {
    return otherMass * getSurfaceGravity();
  }

  public double getAngle()
  {
    synchronized (this)
    {
      return m_angle;
    }
  }

  public synchronized void setAngle(double a)
  {
    m_angle = a;
  }

  public static void go()
  {
	if (m_isRunning)
		return;
	boolean alive = true;
	while (alive)
	{
		alive = false;
		for (Planet p : Planet.values())
			alive |= p.m_thread.isAlive();
		try
		{
			Thread.sleep(500);
		}
		catch(Exception ex)
		{
			ex.printStackTrace();
		}
	}
	m_isRunning = true;
	for (Planet p : Planet.values())
	{
		p.m_thread = new Thread(p);
		p.m_thread.start();
	}
  }
  
  public static void stop()
  {
    m_isRunning = false;
  }

  public void run()
  {
    double omega = m_speed / m_distance;
    while (m_isRunning)
    {
      try
      {
        Thread.sleep(1000 / 10);
        synchronized(this)
        {
          m_angle += (180 / Math.PI) * omega * SCALE * 1;
          m_angle %= 360;
          dispatchMove();
        }
      }
      catch(Exception ex)
      {
        ex.printStackTrace();
      }
    }
  }

  public double getX()
  {
    return m_distance * Math.cos(m_angle * Math.PI / 180);
  }

  public double getY()
  {
    return m_distance * Math.sin(m_angle * Math.PI / 180);
  }

}
